<?php
/*
Plugin Name: Filter Main Query
Version: 1.0.0
Description: Filter the main query and RSS feed by appending query args to the URL.
Author: Brian Richards
Author URI: https://WPSessions.com
Plugin URI: https://WPSessions.com
Text Domain: filter-main-query
Domain Path: /languages
*/

/**
 * Register all necesssary scripts and styles.
 *
 * @since 1.0.0
 */
function fmq_register_scripts() {
	wp_register_script( 'select2', plugin_dir_url( __FILE__ ) . 'includes/js/select2.min.js', array( 'jquery' ), '4.0.0', true );
	wp_register_script( 'fmq-front-end', plugin_dir_url( __FILE__ ) . 'includes/js/fmq-front-end.js', array( 'select2' ), '1.0.0', true );
	wp_register_style( 'select2', plugin_dir_url( __FILE__ ) . 'includes/css/select2.min.css' );
}
add_action( 'wp_enqueue_scripts', 'fmq_register_scripts' );

/**
 * Filter the main query (via pre_get_posts) if URL filters are present.
 *
 * @since  1.0.0
 *
 * @param  object $query WP Query object.
 */
function fmq_maybe_filter_query( $query ) {
	if ( $query->is_main_query() && fmq_should_filter_query() ) {
		$filters = fmq_get_applied_filter_taxonomies();
		$query = fmq_filter_query( $query, $filters );
	}
}
add_action( 'pre_get_posts', 'fmq_maybe_filter_query', 10 );

/**
 * Check if filters are present.
 *
 * @since  1.0.0
 *
 * @return bool True if set, otherwise false.
 */
function fmq_should_filter_query() {
	return isset( $_GET['filter'] );
}

function fmq_get_applied_filter_taxonomies() {
	// get all applied query args
	// parse list for those that begin with filter-
	// build array of only taxonomy slugs
	// return final array
}

/**
 * Set taxonomy query based on present filters.
 *
 * @since  1.0.0
 *
 * @param  object $query WP Query object.
 * @return object        WP Query object.
 */
function fmq_filter_query( $query, $filters ) {

	$tax_query = array();

	$filters = fmq_get_filters( $filters );
	foreach( $filters as $taxonomy => $terms ) {
		if ( ! empty( $terms ) ) {
			$tax_query[] = array(
				'taxonomy' => $taxonomy,
				'field'    => 'slug',
				'terms'    => $terms,
			);
		}
	}

	if ( ! empty( $tax_query ) ) {
		$tax_query['relation'] = 'AND';
		$query->set( 'tax_query', $tax_query );
	}

	return $query;
}

/**
 * Get present query filters.
 *
 * @since  1.0.0
 *
 * @return array Present query filters.
 */
function fmq_get_filters( $args = array() ) {

	$args = wp_parse_args( $args, array(
		'taxonomies' => array( 'topic', 'technology' ),
	) );

	$taxonomies = array();

	foreach ( $args['taxonomies'] as $taxonomy ) {
		$taxonomies[ $taxonomy ] = fmq_get_filter( $taxonomy );
	}

	return $taxonomies;
}

/**
 * Get present filter from URL query arg.
 *
 * @since  1.0.0
 *
 * @param  string $filter_type Filter query arg.
 * @return array               Filter terms.
 */
function fmq_get_filter( $filter_type = 'category' ) {
	$filter_key = 'filter-' . sanitize_title( $filter_type );
	return isset( $_GET[ $filter_key ] ) ? explode( ',', esc_attr( urldecode( $_GET[ $filter_key ] ) ) ) : array();
}

/**
 * Get all filter controls for inclusion on a page.
 *
 * @since  1.0.0
 *
 * @param  array  $taxonomies Taxonomies to use as filters.
 * @return string             HTML Markup.
 */
function fmq_get_filter_controls( $taxonomies = array() ) {

	$output = '';

	$output .= '<div class="fmq-filters">';

	foreach( $taxonomies as $taxonomy ) {
		$output .= fmq_get_filter_control( $taxonomy );
	}

	$output .= fmq_get_filter_control_links();
	$output .= '</div><!-- .fmq-filters -->';

	return $output;
}

/**
 * Get control input for a single filter.
 *
 * @since  1.0.0
 *
 * @param  string $taxonomy Taxonomy slug.
 * @return string           HTML Markup.
 */
function fmq_get_filter_control( $taxonomy = 'category' ) {

	$output = '';

	$taxonomy = get_taxonomy( $taxonomy );

	if ( ! is_object( $taxonomy ) ) {
		return;
	}

	$terms = get_terms( $taxonomy->name, array(
		'orderby'    => 'count',
		'order'      => 'DESC',
		'hide_empty' => true,
		'number'     => 200,
	) );

	if ( is_array( $terms ) && ! empty( $terms ) ) {

		wp_enqueue_script( 'fmq-front-end' );
		wp_enqueue_style( 'select2' );

		$options = '';
		foreach ( $terms as $term ) {
			$options .= sprintf(
				'<option %1$s value="%2$s">%3$s (%4$s)</option>',
				selected( in_array( $term->slug, fmq_get_filter( $taxonomy->name ) ), true, false ),
				$term->slug,
				$term->name,
				$term->count
			);
		}

		$output .= sprintf(
			'<div class="fmq-filter-wrap fmq-%1$s-wrap">
			<label for="fmq-filter-%1$s">%2$s</label>
			<select name="fmq-filter-%1$s" data-placeholder="%3$s" class="fmq-filter fmq-filter-%1$s" multiple="multiple">
			%4$s
			</select>
			</div><!-- .fmq-filter-wrap -->',
			esc_attr( $taxonomy->name ),
			sprintf( __( 'Limit to these %s:', 'fmq' ), $taxonomy->labels->name ),
			sprintf( __( 'Select %s&hellip;', 'fmq' ), $taxonomy->labels->name ),
			$options
		);
	}

	return $output;
}

/**
 * Get links for applying/bookmarking filters.
 *
 * @since  1.0.0
 *
 * @return string HTML Markup.
 */
function fmq_get_filter_control_links() {
	$output = '';
	$output .= '<div class="fmq-filter-wrap fmq-links-wrap">';

	$output .= sprintf(
		'<a href="#" data-base-url="%1$s" class="fmq-apply-filters button">%2$s</a>',
		fmq_get_base_url(),
		__( 'Update Results', 'fmq' )
	);

	$output .= sprintf(
		'<a href="#" data-base-url="%1$s" class="fmq-rss">%2$s</a>',
		get_feed_link(),
		__( 'Filtered RSS Feed <i class="fa fw fa-rss"></i>', 'fmq' )
	);

	$output .= '</div><!-- .fmq-filter-wrap -->';
	return $output;
}

/**
 * Get filter base URL.
 *
 * @since  1.0.0
 *
 * @return string URL.
 */
function fmq_get_base_url() {
	global $wp;
	return preg_replace( '/page\/\d+\/?/', '', home_url( add_query_arg( array(), $wp->request ) ) );
}
